@@ -26,8 +26,22 @@ from typing import (
     overload,
 )
 
-from beet import Cache, Context, DataPack, Function, TextFileBase
-from beet.core.utils import FileSystemPath, JsonDict, extra_field, import_from_string
+from beet import (
+    LATEST_MINECRAFT_VERSION,
+    Cache,
+    Context,
+    DataPack,
+    Function,
+    NamespaceFile,
+    TextFileBase,
+)
+from beet.core.utils import (
+    FileSystemPath,
+    JsonDict,
+    VersionNumber,
+    extra_field,
+    import_from_string,
+)
 from pydantic import BaseModel
 from tokenstream import InvalidSyntax, TokenStream
 from tokenstream.location import set_location
@@ -45,7 +59,6 @@ from .dispatch import Dispatcher, MutatingReducer, Reducer
 from .parse import delegate, get_parsers
 from .serialize import Formatting, Serializer
 from .spec import CommandSpec
-from .utils import VersionNumber
 
 AstNodeType = TypeVar("AstNodeType", bound=AstNode)
 TextFileType = TypeVar("TextFileType", bound=TextFileBase[Any])
@@ -85,7 +98,7 @@ class AstCacheBackend:
 class MechaOptions(BaseModel):
     """Mecha options."""
 
-    version: VersionNumber = "1.18"
+    version: str = ""
     multiline: bool = False
     formatting: Formatting = "dense"
     readonly: Optional[bool] = None
@@ -99,7 +112,7 @@ class Mecha:
     """Class exposing the command api."""
 
     ctx: InitVar[Optional[Context]] = None
-    version: InitVar[VersionNumber] = "1.18"
+    version: InitVar[VersionNumber] = LATEST_MINECRAFT_VERSION
     multiline: InitVar[bool] = False
     formatting: InitVar[Formatting] = "dense"
     readonly: bool = False
@@ -111,6 +124,10 @@ class Mecha:
 
     spec: CommandSpec = extra_field(default=None)
 
+    providers: List[Type[NamespaceFile]] = extra_field(
+        default_factory=lambda: [Function]
+    )
+
     lint: Dispatcher[AstRoot] = extra_field(init=False)
     transform: Dispatcher[AstRoot] = extra_field(init=False)
     optimize: Dispatcher[AstRoot] = extra_field(init=False)
@@ -134,7 +151,7 @@ class Mecha:
     ):
         if ctx:
             opts = ctx.validate("mecha", MechaOptions)
-            version = opts.version
+            version = opts.version or ctx.minecraft_version
             multiline = opts.multiline
             formatting = opts.formatting
 
@@ -371,17 +388,20 @@ class Mecha:
             if match is None:
                 match = self.match
 
-            for key in source.functions.match(*match or ["*"]):
-                value = source.functions[key]
-                self.database[value] = CompilationUnit(
-                    resource_location=key,
-                    filename=(
-                        os.path.relpath(value.source_path, self.directory)
-                        if value.source_path
-                        else None
-                    ),
-                )
-                self.database.enqueue(value)
+            for file_type in self.providers:
+                if not issubclass(file_type, TextFileBase):
+                    continue
+                for key in source[file_type].match(*match or ["*"]):
+                    value = source[file_type][key]
+                    self.database[value] = CompilationUnit(
+                        resource_location=key,
+                        filename=(
+                            os.path.relpath(value.source_path, self.directory)
+                            if value.source_path
+                            else None
+                        ),
+                    )
+                    self.database.enqueue(value)
         else:
             if isinstance(source, (list, str)):
                 source = Function(source)
@@ -424,7 +444,8 @@ class Mecha:
                     continue
                 with self.steps[step].use_diagnostics(compilation_unit.diagnostics):
                     compilation_unit.ast = self.steps[step](compilation_unit.ast)
-                    self.database.enqueue(function, step + 1)
+                    if compilation_unit.ast:
+                        self.database.enqueue(function, step + 1)
 
             elif not readonly:
                 if not compilation_unit.ast:
